/**
 * Created by lejard_h on 04/01/16.
 */

library serializer.test;

import "dart:convert";
import "package:test/test.dart";
import "package:serializer/serializer.dart";

abstract class DontWantToBeSerialize {
  String foo = "bar";
}

@serializable
abstract class Proxy extends Serialize {}

abstract class ProxyA extends Proxy {}

@serializable
class ModelInt extends ProxyA {
  int _bar;


  int get bar => _bar;
  set bar(int value) => _bar = value;

  ModelInt([this._bar = 42]);
}

@serializable
class ModelA extends ProxyA {
  String _foo;


  String get foo => _foo;
  set foo(String value) => _foo = value;

  ModelA([this._foo = "bar"]);
}

class Test {
  String toto = "tata";
}

@serializable
class ModelB extends ProxyA {
  Test foo = new Test();
  String toto = "tata";
}

@serializable
class ModelC extends ProxyA {
  ModelA foo = new ModelA();
  String plop = "titi";
}

@serializable
class ModelD extends ProxyA {
  List<ModelA> tests;

  ModelD([this.tests]);
}

@serializable
class ModelE extends ProxyA {
  List<String> tests;

  ModelE([this.tests]);
}

@serializable
class WithIgnore extends ProxyA {
  String a;
  String b;
  @ignore
  String secret;

  WithIgnore([this.a, this.b, this.secret]);
}

@serializable
class Date extends ProxyA {
  DateTime date = new DateTime.now();

  Date([this.date]);
}

@serializable
class TestMaxSuperClass extends DontWantToBeSerialize {
  String serialize = "okay";
}

main() {
  initSerializer(type_info_key: "@dart_type");


  group("Serialize", () {
    test("simple test", () {
      ModelA a = new ModelA("toto");

      expect("{@dart_type: ModelA, foo: toto}", a.toString());
      expect('{"@dart_type":"ModelA","foo":"toto"}', a.toJson());
      expect({"@dart_type": "ModelA", "foo": "toto"}, a.toMap());
    });

    test("Map to Map", () {
      Map a = {"test": "toto", "titi": new ModelA()};
      String json = Serializer.toJson(a);
      String mapType = (new Map()).runtimeType.toString();
      expect('{"test":"toto","titi":{"@dart_type":"ModelA","foo":"bar"},"@dart_type":"' + mapType + '"}', json);
    });

    test("list", () {
      List<ModelA> list = [new ModelA("toto"), new ModelA()];

      String json = Serializer.toJson(list);
      expect(
          '["{\\"@dart_type\\":\\"ModelA\\",\\"foo\\":\\"toto\\"}","{\\"@dart_type\\":\\"ModelA\\",\\"foo\\":\\"bar\\"}"]',
          json);
    });

    test("inner list 1", () {
      List<ModelA> list = [new ModelA("toto"), new ModelA()];
      ModelD test = new ModelD(list);
      String json = test.toJson();

      expect(
          '{"@dart_type":"ModelD","tests":["{\\"@dart_type\\":\\"ModelA\\",\\"foo\\":\\"toto\\"}","{\\"@dart_type\\":\\"ModelA\\",\\"foo\\":\\"bar\\"}"]}',
          json);
    });

    test("inner list 2", () {
      ModelE e = new ModelE(["toto","bar"]);

      expect('{"@dart_type":"ModelE","tests":["toto","bar"]}', e.toJson());
    });

    test("inner class non-serializable", () {
      ModelB b = new ModelB();
      expect('{"@dart_type":"ModelB","toto":"tata"}', b.toJson());
      expect({"@dart_type": "ModelB", "toto": "tata"}, b.toMap());
    });

    test("inner class serializable", () {
      ModelC c = new ModelC();
      expect(
          '{"@dart_type":"ModelC","foo":{"@dart_type":"ModelA","foo":"bar"},"plop":"titi"}',
          c.toJson());
      expect({
        "@dart_type": "ModelC",
        "foo": {"@dart_type": "ModelA", "foo": "bar"},
        "plop": "titi"
      }, c.toMap());
    });

    test("list class non-serializable", () {
      List list = [new ModelB(), new ModelB()];
      String json = Serializer.toJson(list);
      expect(
          '["{\\"@dart_type\\":\\"ModelB\\",\\"toto\\":\\"tata\\"}","{\\"@dart_type\\":\\"ModelB\\",\\"toto\\":\\"tata\\"}"]',
          json);
    });

    test("list inner list", () {
      List listA = [new ModelB(), new ModelB()];
      List listB = [new ModelB(), listA];
      String json = Serializer.toJson(listB);
      print(json);
      expect(
          '["{\\"@dart_type\\":\\"ModelB\\",\\"toto\\":\\"tata\\"}",["{\\"@dart_type\\":\\"ModelB\\",\\"toto\\":\\"tata\\"}","{\\"@dart_type\\":\\"ModelB\\",\\"toto\\":\\"tata\\"}"]]',
          json);
    });

    test("list class serializable", () {
      List list = [new ModelC(), new ModelC()];
      String json = Serializer.toJson(list);
      expect(
          '["{\\"@dart_type\\":\\"ModelC\\",\\"foo\\":{\\"@dart_type\\":\\"ModelA\\",\\"foo\\":\\"bar\\"},\\"plop\\":\\"titi\\"}","{\\"@dart_type\\":\\"ModelC\\",\\"foo\\":{\\"@dart_type\\":\\"ModelA\\",\\"foo\\":\\"bar\\"},\\"plop\\":\\"titi\\"}"]',
          json);
    });

    test("Datetime", () {
      Date date = new Date(new DateTime(2016, 1, 1));
      expect({"@dart_type": "Date", "date": "2016-01-01T00:00:00.000"},
          date.toMap());
      expect('{"@dart_type":"Date","date":"2016-01-01T00:00:00.000"}',
          date.toJson());
    });

    test("Max Superclass", () {
      TestMaxSuperClass _test = new TestMaxSuperClass();
      expect(
          '{"@dart_type":"TestMaxSuperClass","serialize":"okay"}',
          Serializer.toJson(_test));
      expect({"@dart_type":"TestMaxSuperClass","serialize":"okay"}, Serializer.toMap(_test));
    });

    test("Ignore attribute", () {
      WithIgnore _ignore = new WithIgnore("1337", "42", "ThisIsASecret");
      expect(
          '{"@dart_type":"WithIgnore","a":"1337","b":"42"}',
          Serializer.toJson(_ignore));
      expect({"@dart_type":"WithIgnore","a":"1337","b":"42"}, Serializer.toMap(_ignore));
    });
  });

  group("Deserialize", () {
    test("simple test - fromJson", () {
      ModelA a =
      Serializer.fromJson('{"@dart_type":"ModelA","foo":"toto"}', ModelA);

      expect(ModelA, a.runtimeType);
      expect("toto", a.foo);
    });

    test("simple test - fromMap", () {
      ModelA a =
      Serializer.fromMap({"@dart_type": "ModelA", "foo": "toto"}, ModelA);

      expect(ModelA, a.runtimeType);
      expect("toto", a.foo);
    });

    test("Map fromMap Map", () {
      //TODO: add model inside map
      Map a = {"test": "toto"};
      Map b = Serializer.fromMap(a, Map);

      expect(a["test"], b["test"]);
    });

    test("list - fromJson", () {
      List list = Serializer.fromJson(
          '["{\\"@dart_type\\":\\"ModelA\\",\\"foo\\":\\"toto\\"}","{\\"@dart_type\\":\\"ModelA\\",\\"foo\\":\\"bar\\"}"]',
          ModelA);

      expect(2, list.length);
      expect("toto", list[0]?.foo);
      expect("bar", list[1]?.foo);

      expect(ModelA, list[0]?.runtimeType);
      expect(ModelA, list[1]?.runtimeType);
    });

    test("list - fromList", () {
      List list = Serializer.fromList(
          JSON.decode(
              '["{\\"@dart_type\\":\\"ModelA\\",\\"foo\\":\\"toto\\"}","{\\"@dart_type\\":\\"ModelA\\",\\"foo\\":\\"bar\\"}"]'),
          ModelA);

      expect(2, list.length);
      expect("toto", list[0]?.foo);
      expect("bar", list[1]?.foo);

      expect(ModelA, list[0]?.runtimeType);
      expect(ModelA, list[1]?.runtimeType);
    });

    test("inner list 1", () {
      ModelD test = Serializer.fromJson(
          '{"@dart_type":"ModelD","tests":["{\\"@dart_type\\":\\"ModelA\\",\\"foo\\":\\"toto\\"}","{\\"@dart_type\\":\\"ModelA\\",\\"foo\\":\\"bar\\"}"]}',
          ModelD);

      expect(2, test?.tests?.length);
      expect(ModelA, test?.tests[0]?.runtimeType);
      expect(ModelA, test?.tests[1]?.runtimeType);
      expect("toto", test?.tests[0]?.foo);
      expect("bar", test?.tests[1]?.foo);
    });

    test("inner list 2", () {
      ModelE test = Serializer.fromJson(
          '{"tests":["toto","bar"]}',
          ModelE);

      expect(2, test?.tests?.length);
      expect("toto", test?.tests[0]);
      expect("bar", test?.tests[1]);
    });

    test("inner class non-serializable", () {
      ModelB b = Serializer.fromJson(
          '{"@dart_type":"ModelB","toto":"tata","foo":{"toto":"tata"}}',
          ModelB);

      expect(null, b.foo);
    });

    test("inner class serializable", () {
      ModelC c = Serializer.fromJson(
          '{"@dart_type":"ModelC","foo":{"@dart_type":"ModelA","foo":"toto"},"plop":"bar"}',
          ModelC);
      expect(ModelA, c.foo.runtimeType);
      expect("toto", c.foo.foo);
      expect("bar", c.plop);
    });

    test("Datetime", () {
      Date date = Serializer.fromJson(
          '{"@dart_type":"Date","date":"2016-01-01T00:00:00.000"}', Date);

      expect("2016-01-01T00:00:00.000", date.date.toIso8601String());
      expect('{"@dart_type":"Date","date":"2016-01-01T00:00:00.000"}',
          date.toJson());
    });

    test("Max Superclass", () {
      TestMaxSuperClass _test = Serializer.fromJson(
          '{"@dart_type":"TestMaxSuperClass","serialize":"okay","foo":"nobar"}',
          TestMaxSuperClass);

      expect("okay", _test.serialize);
      expect("bar", _test.foo);
    });

    test("Ignore attribute", () {
      WithIgnore _ignore = Serializer.fromJson(
          '{"@dart_type":"WithIgnore","a":"1337","b":"42","secret":"ignore"}',
          WithIgnore);

      expect("1337", _ignore.a);
      expect("42", _ignore.b);
      expect(null, _ignore.secret);
    });
  });
}
